
from functools import wraps
from sanic import Blueprint, Sanic
from co6co.utils import log
from sanic.request import Request
from ...services.configCache import ConfigCache


def ConfigEntry(f):
    """
    缓存配置相关
    """
    @wraps(f)
    async def _function(*args, **kwargs):
        for arg in args:
            if isinstance(arg, Request):
                cacheManage = ConfigCache(arg)
                break
        code = kwargs.get("code", None)
        value = await f(*args, **kwargs)
        if code != None and "SYS_CONFIG" in code:
            if cacheManage != None:
                value = await f(*args, **kwargs)
                cacheManage.setConfig(code, value)
            else:
                log.warn("cacheManage 未找到 Request 参数")
        return value

    return _function




from sanic.response import text
from sanic import Request
from co6co_sanic_ext.utils import JSON_util
from co6co_sanic_ext.model.res.result import Result
from sqlalchemy.ext.asyncio import AsyncSession

from sqlalchemy.sql import Select, or_, and_, text as sqlText
from co6co_db_ext.db_utils import db_tools
from co6co_web_db.view_model import BaseMethodView, Request
from ..base_view import CtxMethodView
from ...model.enum import menu_type
from ...model.pos.right import menuPO, MenuRolePO, UserPO, UserGroupRolePO, UserRolePO
from ...model.filters.menu_filter import menu_filter
from ...services import getCurrentUserId
from co6co.utils import log


class ui_tree_view(CtxMethodView):
    routePath = "/tree/"

    async def get(self, request: Request):
        """
        树形选择下拉框数据
        selectTree :  el-Tree
        """
        queryRoleSml = sqlText(
            """
            (select  g.role_id from sys_user_group_role g
            INNER JOIN sys_user u
            on g.user_group_id=u.user_group_id
            where u.id=:userId)
            UNION
            (select ur.role_id from sys_user_role ur
            INNER JOIN sys_user u
            on ur.user_id=u.id
            where u.id=:userId)
            """
        )
        currentId = getCurrentUserId(request)
        userRolesSelect = (
            Select(UserRolePO.roleId).filter(UserRolePO.userId == UserPO.id, UserPO.id == currentId)
        )
        userGroupRolesSelect = (
            Select(UserGroupRolePO.roleId).filter(UserGroupRolePO.userGroupId == UserPO.userGroupId, UserPO.id == currentId)
        )
        # roleList=await self._query(request,queryRoleSml,isPO=False,param={"userId":1})
        roleList = await self._query(request, userRolesSelect.union(userGroupRolesSelect), isPO=False)
        #log.warn(roleList)
        roleList = [d.get("role_id") for d in roleList]
        select = (
            Select(menuPO.id, menuPO.category, menuPO.parentId, menuPO.name, menuPO.code, menuPO.icon,  menuPO.url, menuPO.component, menuPO.permissionKey, menuPO.methods)
            .join(MenuRolePO, onclause=MenuRolePO.menuId == menuPO.id)
            .filter(and_(or_(menuPO.category.__eq__(menu_type.group.val), menuPO.category.__eq__(menu_type.subView.val), menuPO.category.__eq__(menu_type.view.val), menuPO.category.__eq__(menu_type.button.val)), MenuRolePO.roleId.in_(roleList)))
            .order_by(menuPO.parentId.asc(), menuPO.order.asc())
        ).distinct(menuPO.id)

        return await self.query_tree(request, select, rootValue=0, pid_field='parentId', id_field="id", isPO=False)

    async def post(self, request: Request):
        """
        树形 table数据
        tree 形状 table
        """
        param = menu_filter()
        param.__dict__.update(request.json)
        if len(param.filter()) > 0:
            return await self.query_page(request, param)
        return await self.query_tree(request, param.create_List_select(), rootValue=0, pid_field='parentId', id_field="id")



from sanic import Request
from co6co_sanic_ext.model.res.result import Result
from ..base_view import AuthMethodView
import os
from sanic.response import file_stream, json
from co6co.utils import log 
import mimetypes

class RenameView(AuthMethodView):
    routePath = "/rename"

    async def post(self, request: Request):
        """
        重命名 文件或者目录
        @param {path:D:\\abcc\\xxx.xx,name:xxx.txt}
        """
        path_param = request.json.get("path", None)
        name = request.json.get("name", None)
        if path_param is None or name is None:
            return self.response_json(Result.fail(message="path和name参数是必须的"))
        try:
            os.renames(path_param, os.path.join(os.path.dirname(path_param), name))
            return self.response_json(Result.success(message="重命名完成！"))
        except Exception as e:
            return self.response_json(Result.fail(message="重命名失败{}".format(e)))


class NewFolderView(AuthMethodView):
    routePath = "/new"

    async def post(self, request: Request):
        """
        新建文件夹
        @param {path:D:\\abcc\\xxx.xx,name:xxx.txt}
        """
        path_param = request.json.get("path", None)
        name = request.json.get("name", None)
        if path_param is None or name is None:
            return self.response_json(Result.fail(message="path和name参数是必须的"))
        try:
            os.makedirs(os.path.join(path_param, name))
            return self.response_json(Result.success())
        except Exception as e:
            return self.response_json(Result.fail(message="新建文件夹'{}'失败:{}".format(name, e)))


class FileContentView(AuthMethodView):
    routePath = "/file"

    async def post(self, request: Request):
        """
        获取文件内容
        @param {path:D:\\abcc\\xxx.xx }
        """
        try: 
            path_param=None
            log.warn(request.json)
            path_param = request.json.get("path", None)
            if path_param is None:
                return self.response_json(Result.fail(message="path参数是必须的"))
            if os.path.exists(path_param) and os.path.isfile(path_param):
                return await file_stream(path_param,mime_type=mimetypes.guess_type(path_param)[0] or 'application/octet-stream')
            else:
                return json({}, 404)
        except Exception as e: 
            return self.response_json(Result.fail(message="获取文件内容'{}'失败:{}".format(path_param, e)))



from __future__ import annotations
from co6co_db_ext.po import BasePO, TimeStampedModelPO, UserTimeStampedModelPO, CreateUserStampedModelPO
from sqlalchemy import func, INTEGER, Integer, UUID,  INTEGER, BigInteger, Column, ForeignKey, String, DateTime
from sqlalchemy.orm import relationship, declarative_base, Relationship
import co6co.utils as tool
from co6co.utils import hash
import sqlalchemy
from sqlalchemy.schema import DDL
from sqlalchemy import MetaData
import uuid
from ..enum import user_category, user_state

# metadata = MetaData()
# BasePO = declarative_base(metadata=metadata)


class UserPO(UserTimeStampedModelPO):
    __tablename__ = "sys_user"
    id = Column("id", BigInteger, comment="主键", autoincrement=True, primary_key=True)
    userName = Column("user_name", String(64), unique=True,  comment="userName")
    category = Column("category", Integer, comment=f"用户类别:{user_category.to_labels_str()}")
    password = Column("user_pwd", String(256), comment="密码")
    salt = Column("user_salt", String(64), comment="pwd=盐")
    userGroupId = Column("user_group_id", ForeignKey("sys_user_group.id", ondelete="CASCADE"))
    state = Column("state", INTEGER, comment=f"用户状态:{user_state.to_labels_str()}",  server_default='0', default=0)
    lockTime = Column("lock_time", DateTime, comment="锁定时间")
    avatar = Column("avatar", String(256), comment="图像URL")
    remark = Column("remark", String(1500), comment="备注")

    accountPOs = Relationship("AccountPO", back_populates="userPO", uselist=True, passive_deletes=True)
    userGroupPO = Relationship("UserGroupPO", back_populates="userPOs")
    rolePOs = Relationship("RolePO", secondary="sys_user_role", back_populates="userPOs", passive_deletes=True)

    def update(self, po: UserPO):
        self.userName = po.userName
        self.category = po.category
        self.userGroupId = po.userGroupId
        self.state = po.state
        self.remark = po.remark

    @staticmethod
    def generateSalt() -> str:
        """
        生成salt
        """
        return tool.getRandomStr(6)

    def encrypt(self, password: str = None) -> str:
        """
        加密密码
        不保存到属性
        """
        if password != None:
            return hash.md5(self.salt+password)
        return hash.md5(self.salt+self.password)

    def verifyPwd(self, plainText: str) -> bool:
        """
        验证输入的密码是否正确
        """
        return hash.md5(self.salt+plainText) == self.password

    def to_jwt_dict(self):
        """
        jwt 保存内容
        """
        return {"id": self.id, "userName": self.userName, "group_id": self.userGroupId}


class AccountPO(UserTimeStampedModelPO):
    """
    账号：以各种方式登录系统
    用户名  EMAIL,openID
    """
    __tablename__ = "sys_account"
    uid = Column("uuid", String(36),  primary_key=True, default=uuid.uuid4())
    userId = Column("user_id", ForeignKey("{}.{}".format(UserPO.__tablename__, UserPO.id.name), ondelete="CASCADE"), nullable=False, index=True, unique=True)
    accountName = Column("account_name", String(64), unique=True)
    attachInfo = Column("attach_info", String(255),
                        comment="有些信息需要存下来，才能配合账号使用")
    category = Column("category", INTEGER)
    status = Column("status", String(16), comment="状态,根据账号类型，有不同得解释")
    userPO = Relationship(UserPO, back_populates="accountPOs")


class UserGroupPO(UserTimeStampedModelPO):
    """
    用户组
    """
    __tablename__ = "sys_user_group"
    id = Column("id", Integer, autoincrement=True, primary_key=True)
    parentId = Column("parent_id", Integer)
    name = Column("name", String(64))
    code = Column("code", String(64), unique=True)
    order = Column("order", Integer)
    userPOs = Relationship(
        "UserPO", back_populates="userGroupPO", uselist=True, passive_deletes=True)

    def update(self, po: UserGroupPO):
        self.code = po.code
        self.name = po.name
        self.parentId = po.parentId
        self.order = po.order


class RolePO(TimeStampedModelPO):
    """
    角色表
    """
    __tablename__ = "sys_role"

    id = Column("id", BigInteger, comment="主键",
                autoincrement=True, primary_key=True)
    name = Column("name", String(64), comment="角色名")
    code = Column("code", String(64), unique=True)
    order = Column("order", Integer)
    remark = Column("remark", String(255), comment="备注")

    userPOs = Relationship("UserPO", secondary="sys_user_role",
                           back_populates="rolePOs", passive_deletes=True)
    menuPOs = Relationship("menuPO", secondary="sys_menu_role",
                           back_populates="rolePOs", passive_deletes=True)

    def update(self, po: RolePO):
        self.code = po.code
        self.name = po.name
        self.remark = po.remark
        self.order = po.order


class UserRolePO(CreateUserStampedModelPO):
    """
    用户_角色表
    """
    __tablename__ = "sys_user_role"

    userId = Column("user_id", ForeignKey("{}.{}".format(UserPO.__tablename__, UserPO.id.name)),   comment="主键id", primary_key=True)
    roleId = Column("role_id", ForeignKey("{}.{}".format(RolePO.__tablename__, RolePO.id.name)),   comment="主键id", primary_key=True)


class UserGroupRolePO(CreateUserStampedModelPO):
    """
    用户组_角色表
    """
    __tablename__ = "sys_user_group_role"

    userGroupId = Column("user_group_id", ForeignKey("{}.{}".format(UserGroupPO.__tablename__, UserGroupPO.id.name)),   comment="主键id", primary_key=True)
    roleId = Column("role_id", ForeignKey("{}.{}".format(RolePO.__tablename__, RolePO.id.name)),   comment="主键id", primary_key=True)


class menuPO(UserTimeStampedModelPO):
    '''
    菜单表
    '''
    __tablename__ = "sys_menu"

    id = Column("id", Integer, comment="主键",
                autoincrement=True, primary_key=True)
    name = Column("name", String(64), comment="名称")
    code = Column("code", String(64), unique=True,  comment="code")
    parentId = Column("parent_id", Integer)
    category = Column("category", Integer, comment='0:后台URL,1:view,2:button')
    icon = Column("icon", String(128))
    url = Column("url", String(255))
    component = Column("component", String(128))
    methods = Column("methods", String(
        64), comment='方法名:GET|POST|DELETE|PUT|PATCH')
    permissionKey = Column("permission_key", String(64),
                           comment="view 与button 有效")
    order = Column("order", Integer)
    status = Column("status", Integer)
    remark = Column("remark", String(255), comment="备注")

    rolePOs = Relationship("RolePO", secondary="sys_menu_role",
                           back_populates="menuPOs", passive_deletes=True)

    def update(self, po: menuPO):
        self.code = po.code
        self.name = po.name
        self.parentId = po.parentId
        self.category = po.category
        self.icon = po.icon
        self.url = po.url
        self.methods = po.methods
        self.permissionKey = po.permissionKey
        self.component = po.component
        self.order = po.order
        self.status = po.status
        self.remark = po.remark


class MenuRolePO(CreateUserStampedModelPO):
    """
    权限_角色表
    """
    __tablename__ = "sys_menu_role"

    menuId = Column("menu_id", ForeignKey("{}.{}".format(menuPO.__tablename__, menuPO.id.name), ondelete="CASCADE"),   comment="主键id", primary_key=True)
    roleId = Column("role_id", ForeignKey("{}.{}".format(RolePO.__tablename__, RolePO.id.name), ondelete="CASCADE"),   comment="主键id", primary_key=True)


class LoginLogPO(CreateUserStampedModelPO):
    """
    登录日志
    """
    __tablename__ = "sys_login_log"
    id = Column("id", Integer, comment="主键",
                autoincrement=True, primary_key=True)
    name = Column("name", String(64), comment="名称")
    state = Column("state", String(64),   comment="状态")
    ipAddress = Column("ip_address", String(64))
    message = Column("message", String(254))


class OperationLogPO(CreateUserStampedModelPO):
    __tablename__ = "sys_operation_log"
    id = Column("id", Integer, comment="主键",
                autoincrement=True, primary_key=True)
    category = Column("category", String(64), comment="类型")
    name = Column("name", String(64), comment="名称")  # 修改用户
    state = Column("state", String(64),    comment="状态")
    target = Column("target", String(
        64),   comment="修改目标,数据表中的某个关键字段[eg. admin 用户]")
    data = Column("data", String(2048), comment="数据")


'''
class data_permission_base(UserTimeStampedModelPO):
    """
    数据权限
    1. 根据具体业务创建相关人员或组字段，以满足需求
    """
    createUser= Column("create_user", BigInteger,comment="创建人")  
    ownUser= Column("own_user", BigInteger,comment="所有者")  
    ownGroup= Column("own_group", BigInteger,comment="所属组")  
    ownSales= Column("own_sales", BigInteger,comment="所属销售")
    ownMaintain= Column("own_maintain", BigInteger,comment="所属维护")
    
'''




from sanic import Request
from sqlalchemy.sql import Select
from co6co.utils import log
from sqlalchemy.ext.asyncio import AsyncSession
from .baseCache import BaseCache
from co6co_db_ext.db_utils import DbCallable, db_tools
from ..model.pos.other import sysConfigPO
import json as sysJson
from typing import TypedDict


async def get_config(request: Request, code: str, *, useDist=False, default: any = None) -> str | dict | None:
    """
    获取配置
    """
    try:
        cache = ConfigCache(request)
        result = cache.getConfig(code)
        if result == None:
            await cache.queryConfig(code)
            result = cache.getConfig(code)
        if result and useDist:
            result = sysJson.loads(result)
            # 　如果default是dict,则合并，
            result = default.update(result) if default and isinstance(default, dict) else result
        if result == None:
            log.warn("未找到'{}'的相关配置,使用默认配置：{}".format(code, default))
            result = default
        return result
    except Exception as e:
        log.error("获取配置失败:{},使用默认配置：{}".format(e, default))
        return default


async def get_upload_path(request: Request) -> str:
    """
    获取上传路径
    数据库未配置使用 /upload
    """
    key = "SYS_CONFIG_UPLOAD_PATH"
    return await get_config(request, key, default="/upload")


async def get_terminal_access_token(request: Request) -> str:
    """
    获取终端配置
    """
    cache = ConfigCache(request)
    cache.setCache("SYS_CONFIG_TERMINAL_ACCESS_TOKEN", "123456")
    # 获取
    return cache.getConfig("SYS_CONFIG_TERMINAL_ACCESS_TOKEN")


class UserConfig(TypedDict):
    loginFail: int
    lockSeconds: int


async def get_user_config(request: Request) -> UserConfig:
    key = "SYS_USER_CONFIG"
    default = {"loginFail": 3, "lockSeconds": 5*60}
    config = await get_config(request, key, useDist=True, default=default)

    return config


class ConfigCache(BaseCache):

    def __init__(self, request: Request) -> None:
        super().__init__(request)

    @property
    def configKeyPrefix(self):
        return 'ConfigKey'

    def getKey(self, code: str):
        """
        获取Key
        """
        return "{}_{}".format(self.configKeyPrefix, code)

    async def queryConfig(self, code: str) -> str | None:
        """
        查询当前用户的所拥有的角色
        结果放置在cache中
        """
        callable = DbCallable(self.session)

        async def exe(session) -> str | None:
            select = (
                Select(sysConfigPO.name, sysConfigPO.code,
                       sysConfigPO.value, sysConfigPO.remark)
                .filter(sysConfigPO.code.__eq__(code))
            )
            data: dict | None = await db_tools.execForMappings(session, select, queryOne=True)
            result = None
            if data == None:
                log.warn("query {} config is NULL".format(code))
            else:
                result = data.get("value")
                # 配置的是否时json字符串
                if "{" in result and "}" in result:
                    try:
                        result = sysJson.loads(result)
                    except:
                        log.err("load json config error:{},check config key :{}".format(result, code))
                        result = None
                self.setConfig(code, result)
            return result

        return await callable(exe)

    def setConfig(self, code: str, value: str):
        if code != None:
            self.setCache(self.getKey(code), value)

    def getConfig(self, code: str) -> str | None:
        if code != None:
            return self.getCache(self.getKey(code))

    def clear(self, code: str) -> str | None:
        if code != None:
            return self.remove(self.getKey(code))




from functools import wraps
from sanic.request import Request
from ...model.pos.right import LoginLogPO


from datetime import datetime
from sanic.response import JSONResponse

from co6co.utils import log
from co6co_sanic_ext.model.res.result import Result
from co6co_db_ext.db_utils import db_tools, InsertCallable
from ...services import getCurrentUserId
import json


async def _loginLog(response: JSONResponse, request: Request):
    try:
        po = LoginLogPO()
        po.ipAddress = request.client_ip  # p.ip=self.forwarded['for']
        po.createTime = datetime.now()
        res = json.loads(str(response.body, encoding='utf-8'))
        result = Result.success()
        result.__dict__.update(res)
        po.name = request.json.get("userName")
        if result.code == 0:
            po.createUser = getCurrentUserId(request)
            po.state = "成功"
        else:
            po.state = "失败"
        # log.warn(po.__dict__)
        insert = InsertCallable(request.ctx.session)
        await insert(po)
    except Exception as e:
        log.err("写登录日志失败")


def loginLog(f):
    @wraps(f)
    async def decorated_function(*args, **kwargs):
        request: Request = None
        for a in args:
            if type(a) == Request:
                request = a
            # log.warn("第一个参数",type(a))
        '''
        for a,v in kwargs:
            log.warn("第er个参数",type(a),type(v))
        '''

        response = await f(*args, **kwargs)
        await _loginLog(response, request)
        return response
    return decorated_function



from __future__ import annotations
from co6co_db_ext.po import BasePO, TimeStampedModelPO, UserTimeStampedModelPO, CreateUserStampedModelPO
from sqlalchemy import func, INTEGER, Integer, UUID,  INTEGER, BigInteger, Column, ForeignKey, String, DateTime
from sqlalchemy.orm import relationship, declarative_base, Relationship
import co6co.utils as tool
from co6co.utils import hash
import sqlalchemy
from sqlalchemy.schema import DDL
from sqlalchemy import MetaData
import uuid


class resourcePO(TimeStampedModelPO):
    """
    资源
    """
    __tablename__ = "sys_resource"
    id = Column("id", BigInteger, comment="主键", autoincrement=True, primary_key=True)
    uid = Column("uuid", String(36),  unique=True, default=uuid.uuid1())
    category = Column("category", Integer, comment="资源类型:0:图片资源,1:视频资源, 2:文件")
    subCategory = Column("sub_category", Integer, comment="子资源类型")
    # mysql 56 leng String（256） unique -->Specified key was too long; max key length is 767 bytes
    hash = Column("hash", String(256), unique=True, comment="使用hash256计算值")
    url = Column("url_path", String(255), comment="资源路径,针对根路径下的绝对路径")
    metaName = Column("meta_name", String(255), comment="名称")
    size = Column("size", BigInteger, comment="大小")
    # , passive_deletes=True ->当删除资源是 所有的用户资源已会被删除
    # backref 表示，在 userResourcePO  类中动态创建 userResource 属性，指向当前类。
    # 两个类中显式地使用 back_populates，更显繁琐
    # 不包含外键  默认 uselist=True, 如果是1对1 需要不使用 collection
    userResourceList = Relationship("userResourcePO", back_populates="userResource",  passive_deletes=True)


class userResourcePO(TimeStampedModelPO):
    """
    用户资源资源
    """
    __tablename__ = "user_resource"
    id = Column("id", BigInteger, comment="主键", autoincrement=True, primary_key=True)
    resourceId = Column("resource_id", ForeignKey("{}.{}".format(resourcePO.__tablename__, resourcePO.id.name)))
    ownUserId = Column("own_user_id", BigInteger, comment="所属用户")
    name = Column("name", String(255), comment="名称")
    # 那如果我们需要得知 child 的 parent 对象呢？能不能直接访问 child.parent？
    # 为了实现这个功能，SQLAlchemy 提供了 backref 和 back_populates 两个参数。

    # # 包含 ForeignKey 的类，此属性默认为 attribute，因此不需要 uselist=False
    userResource = Relationship(resourcePO, back_populates="userResourceList")



from co6co_web_db.view_model import BaseMethodView, Request
from .aop.api_auth import authorized, ctx

from co6co_db_ext.db_operations import DbOperations
from co6co_db_ext.db_utils import db_tools
from ..services import getCurrentUserId, getCurrentUserName


class _view():
    def getUserId(self, request: Request):
        """
        获取用户ID
        """
        return getCurrentUserId(request)

    def getUserName(self, request: Request):
        """
        获取当前用户名
        """
        return getCurrentUserName(request)


class CtxMethodView(BaseMethodView, _view):
    decorators = [ctx]


class AuthMethodView(BaseMethodView, _view):
    """
    token 校验 
    api 校验
    """
    decorators = [authorized]



from sanic.request import Request


from sqlalchemy .orm.attributes import InstrumentedAttribute
from typing import Tuple
from co6co_db_ext.db_filter import absFilterItems
from co6co.utils import log
from sqlalchemy import func, or_, and_, Select
from ..pos.right import RolePO
from ...view_model.aop.authonCache import AuthonCacheManage


class role_filter(absFilterItems):
    """
    角色 filter
    """
    name: str = None
    code: str = None
    request: Request = None
    currentRoles: list = None

    def __init__(self, request: Request):
        super().__init__(RolePO)
        self.request = request

    async def init(self):
        cache = AuthonCacheManage(self.request)
        self.currentRoles = await cache.currentRoles
        # 1 超级角色
        if 1 in self.currentRoles:
            self.currentRoles = None

    def filter(self) -> list:
        """
        过滤条件
        """
        filters_arr = []

        if self.checkFieldValue(self.name):
            filters_arr.append(RolePO.name.like(f"%{self.name}%"))
        if self.checkFieldValue(self.code):
            filters_arr.append(RolePO.code.like(f"%{self.code}%"))
        if self.currentRoles != None and len(self.currentRoles) > 0:
            filters_arr.append(RolePO.id.in_(self.currentRoles))

        return filters_arr

    def getDefaultOrderBy(self) -> Tuple[InstrumentedAttribute]:
        """
        默认排序
        """
        return (RolePO.order.asc(),)




from functools import wraps
from sanic import Blueprint, Sanic
from sanic.response import json
from sanic.request import Request

from co6co_web_db.services.jwt_service import validToken, JWT_service
from co6co_sanic_ext.model.res.result import Result
from co6co_sanic_ext.utils import JSON_util
from sqlalchemy.sql import Select
from sqlalchemy.ext.asyncio import AsyncSession
from co6co_db_ext.db_utils import db_tools, DbCallable
from co6co.utils import log

from ...model.pos.right import UserPO, UserRolePO, UserGroupRolePO, menuPO, MenuRolePO
from ...model.enum import menu_type
from .api_check import apiPermissionCheck
from .authonCache import AuthonCacheManage
from ...services.baseCache import AccessTokenCache


async def checkApi(request: Request):
    """
    查询当前用户的对当前API是否有权限
    """
    check = apiPermissionCheck(request)
    await check.init()
    return check.check()


def authorized(f):
    """
    认证
    认证不通过 返回 403,不在执行f 函数
    """
    @wraps(f)
    async def decorated_function(request: Request, *args, **kwargs):
        secret = request.app.config.SECRET
        token = request.token
        valid = False
        # 使用 accessToken 认证
        if token and '.' not in token:
            accessToken = AccessTokenCache(request)
            valid = await accessToken.validAccessToken(token)
        elif token:
            valid = await validToken(request, secret)
        if not valid:
            return JSON_util.response(Result.fail(message="token invalid or expire"), status=403)
        # //dodo debug
        valid = await checkApi(request)
        if valid:
            # the user is authorized.
            # run the handler method and return the response
            response = await f(request, *args, **kwargs)
            return response
        else:
            # the user is not authorized.
            return JSON_util.response(Result.fail(message="not_authorized"), status=403)
    return decorated_function


def ctx(f):
    """
    设置请求上下文
    有token 将可以再上下文中获取 用户Id等信息 
    """
    @wraps(f)
    async def decorated_function(request: Request, *args, **kwargs):
        await validToken(request, request.app.config.SECRET)
        response = await f(request, *args, **kwargs)
        return response

    return decorated_function


def menuChanged(f):
    """
    设置请求上下文
    有token 将可以再上下文中获取 用户Id等信息 
    """
    @wraps(f)
    async def decorated_function(*args, **kwargs):
        for a in args:
            if isinstance(a, Request):
                cacheManage = AuthonCacheManage(a)
                cacheManage.setMenuDataInvalid()
        response = await f(*args, **kwargs)
        return response

    return decorated_function


def userRoleChanged(f):
    """
    设置请求上下文
    有token 将可以再上下文中获取 用户Id等信息 
    """
    @wraps(f)
    async def decorated_function(*args, **kwargs):
        for a in args:
            if isinstance(a, Request):
                cacheManage = AuthonCacheManage(a)
                cacheManage.setRolesInvalid()
                cacheManage.setMenuDataInvalid()
        response = await f(*args, **kwargs)
        return response

    return decorated_function



from sanic.response import text
from sanic import Request
from co6co_sanic_ext.utils import JSON_util
from co6co_sanic_ext.model.res.result import Result
from sqlalchemy.ext.asyncio import AsyncSession

from sqlalchemy.sql import Select, Delete
from co6co_db_ext.db_utils import db_tools
from co6co_web_db.model.params import associationParam


from .base_view import AuthMethodView
from co6co_web_db.view_model import get_one
from .resource import resource_baseView, FileResult
from ..model.pos.right import UserPO, RolePO, UserRolePO, AccountPO
from ..model.filters.user_filter import user_filter


class changePwd_view(AuthMethodView):
    routePath = "/changePwd"

    async def post(self, request: Request):
        """
        修改密码：
        {
            oldPassword:"",
            newPassword:"",
            remark:""
        }
        """
        data = request.json

        userId = self.getUserId(request)
        userName = self.getUserName(request)

        oldPassword = data["oldPassword"]
        password = data["newPassword"]
        remark = data["remark"]
        select = (Select(UserPO).filter(UserPO.userName == userName))

        async def edit(_, one: UserPO):
            if one != None:
                if one.password != one.encrypt(oldPassword):
                    return JSON_util.response(Result.fail(message="输入的旧密码不正确！"))
                if one.encrypt(password) == one.encrypt(oldPassword):
                    return JSON_util.response(Result.fail(message="输入的旧密码与新密码一样！"))
                one.password = one.encrypt(password)
                if remark:
                    one.remark = remark
            return JSON_util.response(Result.success())
        return await self.update_one(request, select, edit)


class user_avatar_view(resource_baseView):
    routePath = "/avatar"

    async def get(self, request: Request):
        """
        获取头像
        """
        userName = self.getUserName(request)
        select = Select(UserPO.avatar).filter(UserPO.userName == userName)
        dicts = await get_one(request, select, isPO=False)

        avatar = dicts.get("avatar")
        if avatar:
            return await self.response_local_file(request, avatar)
        else:
            svg_content = '''
                <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 100 100">
                <!-- 头部 -->
                <circle cx="50" cy="20" r="10" fill="black"/>
                <!-- 身体 -->
                <line x1="50" y1="30" x2="50" y2="70" stroke="black" stroke-width="3"/>
                <!-- 手臂 -->
                <line x1="30" y1="40" x2="70" y2="40" stroke="black" stroke-width="3"/>
                <!-- 腿部 -->
                <line x1="50" y1="70" x2="30" y2="90" stroke="black" stroke-width="3"/>
                <line x1="50" y1="70" x2="70" y2="90" stroke="black" stroke-width="3"/>
                </svg>
            '''
            return text(svg_content, content_type='image/svg+xml')

    async def put(self, request: Request):
        """
        上传图像
        """
        userName = self.getUserName(request)
        select = Select(UserPO).filter(UserPO.userName == userName)
        result = await self.saveFile(request)
        if type(result) == FileResult:

            async def edit(_, one: UserPO):
                if one != None:
                    if result.path:
                        one.avatar = result.path
                return JSON_util.response(Result.success(data=result.path))
            return await self.update_one(request, select, edit)
        else:
            return result


class user_info_view(AuthMethodView):
    routePath = "/currentUser"

    async def get(self, request: Request):
        """
        当前用户信息  
        return {
            data:{
                avatar:""
                remark:""
                userName:""
            } 
        }
        """
        userName = self.getUserName(request)
        select = Select(UserPO.avatar, UserPO.userName, UserPO.remark).filter(UserPO.userName == userName)
        return await self.get_one(request, select, isPO=False)




from functools import wraps
from sanic import Blueprint, Sanic
from co6co.utils import log
from sanic.request import Request
from ...services.baseCache import AccessTokenCache
from typing import Callable


def AccessTokenChange(f: Callable[[Request, str], dict]):
    """
    缓存配置相关
    @param f:方法参数为  Request, token, 返回值:setCurrentUser需要的参数
    setCurrentUser
    """
    @wraps(f)
    async def _function(*args, **kwargs):
        for arg in args:
            if isinstance(arg, Request):
                cacheManage = AccessTokenCache(arg)
                break
        token = kwargs.get("token", None)
        value = await f(*args, **kwargs)
        if value:
            if cacheManage != None:
                value = await f(*args, **kwargs)
                cacheManage.setCache(token, value)
            else:
                log.warn("AccessTokenChange 未找到 Request 参数")
        return value

    return _function




from sanic.response import text
from sanic import Request
from co6co_sanic_ext.utils import JSON_util
from co6co_sanic_ext.model.res.result import Result
from sqlalchemy.ext.asyncio import AsyncSession

from sqlalchemy.sql import Select, Delete
from co6co_db_ext.db_utils import db_tools
from co6co_web_db.model.params import associationParam
from co6co.utils import getRandomStr

from .base_view import AuthMethodView, BaseMethodView
from ..model.pos.right import UserPO, RolePO, UserRolePO, AccountPO
from ..model.enum import user_category
from ..model.filters.user_filter import user_filter
from .aop.api_auth import userRoleChanged
from ..services import generatePageToken, queryUer, getSecret, decodeCode
from .aop.user_aop import AccessTokenChange


@AccessTokenChange
def accessTokenChange(self, token: str, userPo: UserPO = None):
    return userPo.to_jwt_dict()


class user_ass_view(AuthMethodView):
    routePath = "/association/<userId:int>"

    async def post(self, request: Request, userId: int):
        """
        获取用户关联的角色
        """
        subSelect = Select(UserRolePO.roleId, UserRolePO.userId).filter(
            UserRolePO.userId == userId).subquery()
        select = (
            Select(RolePO.id, RolePO.name, RolePO.code,
                   subSelect.c.userId.label("associatedValue"))
            .outerjoin_from(RolePO, subSelect, onclause=subSelect.c.roleId == RolePO.id, full=False)
            .order_by(RolePO.id.asc())
        )
        return await self.query_list(request, select, isPO=False)

    @userRoleChanged
    async def put(self, request: Request, userId: int):
        """
        保存用户关联角色
        """
        param = associationParam()
        param.__dict__.update(request.json)
        currentUserId = self.getUserId(request)
        sml = (
            Delete(UserRolePO).filter(UserRolePO.userId ==
                                      userId, UserRolePO.roleId .in_(param.remove))
        )

        async def createPo(_, roleId: int):
            po = UserRolePO()
            po.roleId = roleId
            po.userId = userId
            return po
        return await self.save_association(request, currentUserId, sml, createPo)


class users_view(AuthMethodView):
    async def get(self, request: Request):
        """
        用户下拉框数据
        selectTree :  el-Tree
        """
        select = (
            Select(UserPO.userName.label("name"), UserPO.id)
            .order_by(UserPO.id.asc())
        )
        return await self.query_list(request, select,  isPO=False)

    async def post(self, request: Request):
        """
        树形 table数据
        tree 形状 table
        """
        param = user_filter()
        param.__dict__.update(request.json)
        return await self.query_page(request, param, isPO=False)

    async def put(self, request: Request):
        """
        增加
        """
        po = UserPO()
        userId = self.getUserId(request)

        async def before(po: UserPO, session: AsyncSession, request):
            exist = await db_tools.exist(session,  UserPO.userName.__eq__(po.userName), column=UserPO.id)
            if exist:
                return JSON_util.response(Result.fail(message=f"'{po.userName}'已存在！"))
            if po.salt == None:
                po.salt = getRandomStr(6)
            if po.category == user_category.normal.val or po.category == user_category.system.val:
                po.password = po.encrypt(po.password)
            else:
                accessTokenChange(po.password, po)
        return await self.add(request, po, userId=userId, beforeFun=before)


class user_view(AuthMethodView):
    routePath = "/<pk:int>"

    async def put(self, request: Request, pk: int):
        """
        编辑
        """
        async def before(oldPo: UserPO, po: UserPO, session: AsyncSession, request):
            exist = await db_tools.exist(session, UserPO.id != oldPo.id, UserPO.userName.__eq__(po.userName), column=UserPO.id)
            if exist:
                return JSON_util.response(Result.fail(message=f"'{po.userName}'已存在！"))

        return await self.edit(request, pk, UserPO, userId=self.getUserId(request), fun=before)

    async def delete(self, request: Request, pk: int):
        """
        删除
        """
        if pk == 1:
            return JSON_util.response(Result.fail(message="不能删除系统默认用户！"))

        async def before(po: UserPO, session: AsyncSession):
            # 用户角色关联
            count = await db_tools.count(session, UserRolePO.userId == po.id, column=UserRolePO.userId)
            if count > 0:
                return JSON_util.response(Result.fail(message=f"该'{po.userName}'用户关联有‘{count}’角色，不能删除！"))
            count = await db_tools.count(session, AccountPO.userId == po.id, column=AccountPO.uid)
            if count > 0:
                return JSON_util.response(Result.fail(message=f"该'{po.userName}'用户关联有‘{count}’账号，不能删除！"))

        return await self.remove(request, pk, UserPO, beforeFun=before)


class sys_users_view(AuthMethodView):
    routePath = "/reset"

    async def post(self, request: Request):
        """
        重置密码
        """
        data = request.json
        userName = data["userName"]
        password = data["password"]
        select = (Select(UserPO).filter(UserPO.userName == userName))
        if userName == None or password == None or len(password) < 6:
            return JSON_util.response(Result.fail(message="请检查提交的用户和密码！"))

        async def edit(_, one: UserPO):
            if one != None:
                if one.salt == None:
                    return JSON_util.response(Result.fail(message=f"用户[{userName}],通过关联创建的用户，完善信息才能重置密码"))
                if one.category == user_category.normal.val or one.category == user_category.system.val:
                    one.password = one.encrypt(password)
                else:
                    one.password = password
                    accessTokenChange(password, one)
                return JSON_util.response(Result.success())
            else:
                return JSON_util.response(Result.fail(message=f"所提供的用户名[{userName}]不存在，请刷新重试！"))

        return await self.update_one(request, select, edit)


class ticketView(BaseMethodView):
    routePath = "/ticket/<code:str>"

    async def get(self, request: Request, code: str):
        """
        通过 code 换取 token
        code 为临时 有过期时间
        """
        userId = await decodeCode(getSecret(request), code)
        if userId == None:
            return JSON_util.response(Result.fail(message="code 无效或已过期"))
        user: UserPO = await queryUer(self.get_db_session(request), userId)
        if user != None:
            token = await generatePageToken(getSecret(request), user)
            return JSON_util.response(Result.success(data=token, message="票据登录成功"))
        else:
            return JSON_util.response(Result.fail(message="未找到所属用户"))





from ..pos.right import menuPO
from sqlalchemy .orm.attributes import InstrumentedAttribute
from typing import Tuple
from co6co_db_ext.db_filter import absFilterItems
from co6co.utils import log
from sqlalchemy import func, or_, and_, Select


class menu_filter(absFilterItems): 
    """
    用户组 filter
    """
    pid:int=None
    name: str = None
    code: str = None 

    def __init__(self): 
        super().__init__(menuPO) 
 
    def filter(self) -> list:
        """
        过滤条件
        """
        filters_arr = []
        if self.checkFieldValue(self.pid) :
            filters_arr.append(menuPO.parentId.__eq__(self.pid))
        if self.checkFieldValue(self.name)  : 
            filters_arr.append(menuPO.name.like(f"%{self.name}%"))
        if self.checkFieldValue(self.code):
            filters_arr.append(menuPO.code.like(f"%{self.code}%")) 
        return filters_arr

   
    def getDefaultOrderBy(self) -> Tuple[InstrumentedAttribute]:
        """
        默认排序
        """
        return (menuPO.order.asc(),)




from sanic.response import text
from sanic import Request
from co6co_sanic_ext.utils import JSON_util
from co6co_sanic_ext.model.res.result import Result
from sqlalchemy.ext.asyncio import AsyncSession

from sqlalchemy.sql import Select, Delete
from co6co_db_ext.db_utils import db_tools
from co6co_web_db.model.params import associationParam

from .base_view import AuthMethodView
from .aop import exist, ObjectExistRoute
from ..model.pos.right import UserGroupPO, RolePO, UserGroupRolePO
from ..model.filters.user_group_filter import user_group_filter
from co6co.utils import log
from .aop.api_auth import userRoleChanged


class user_group_exist_view(AuthMethodView):
    routePath = ObjectExistRoute

    async def get(self, request: Request, code: str, pk: int = 0):
        result = await self.exist(request, UserGroupPO.code == code,
                                  UserGroupPO.id != pk, column=UserGroupPO.id)
        return exist(result, "用户组编辑", code)


class user_group_ass_view(AuthMethodView):
    routePath = "/association/<userGroupId:int>"

    async def post(self, request: Request, userGroupId: int):
        """
        获取用户组关联的角色
        """
        subSelect = Select(UserGroupRolePO.roleId, UserGroupRolePO.userGroupId).filter(
            UserGroupRolePO.userGroupId == userGroupId).subquery()
        select = (
            Select(RolePO.id, RolePO.name, RolePO.code,
                   subSelect.c.roleId.label("associatedValue"))
            .outerjoin_from(RolePO, subSelect, onclause=subSelect.c.roleId == RolePO.id, full=False)
            .order_by(RolePO.name.asc())
        )
        return await self.query_list(request, select, isPO=False)

    @userRoleChanged
    async def put(self, request: Request, userGroupId: int):
        """
        保存角色关联菜单
        """
        param = associationParam()
        param.__dict__.update(request.json)
        userId = self.getUserId(request)

        sml = (
            Delete(UserGroupRolePO).filter(UserGroupRolePO.userGroupId ==
                                           userGroupId, UserGroupRolePO.roleId .in_(param.remove))
        )

        async def createPo(_, roleId: int):
            po = UserGroupRolePO()
            po.roleId = roleId
            po.userGroupId = userGroupId
            return po
        return await self.save_association(request, userId, sml, createPo)


class user_groups_tree_view(AuthMethodView):
    routePath = "/tree"

    async def get(self, request: Request, rootValue: int = None):
        """
        树形选择下拉框数据
        selectTree :  el-Tree
        """
        select = (
            Select(UserGroupPO.id, UserGroupPO.name,
                   UserGroupPO.code, UserGroupPO.parentId)
            .order_by(UserGroupPO.parentId.asc())
        )
        return await self.query_tree(request, select, rootValue=rootValue, pid_field='parentId', id_field="id", isPO=False)

    async def post(self, request: Request):
        """
        树形 table数据
        tree 形状 table
        """
        param = user_group_filter()
        param.__dict__.update(request.json)
        if len(param.filter()) > 0:
            return await self.query_page(request, param)
        return await self.query_tree(request, param.create_List_select(), rootValue=0, pid_field='parentId', id_field="id")


class user_groups_sub_tree_view(user_groups_tree_view):
    routePath = "/tree/<parendId:int>"

    async def get(self, request: Request, parendId: int):
        """
        返回子 树形选择下拉框数据
        """
        return await super().get(request, parendId)


class user_groups_view(AuthMethodView):

    async def get(self, request: Request):
        """
        树形选择下拉框数据
        selectTree :  el-Tree
        """
        select = (
            Select(UserGroupPO.id, UserGroupPO.name,
                   UserGroupPO.code, UserGroupPO.parentId)
            .order_by(UserGroupPO.parentId.asc())
        )
        return await self.query_list(request, select,  isPO=False)

    async def post(self, request: Request):
        """
        树形 table数据
        tree 形状 table
        """
        param = user_group_filter()
        param.__dict__.update(request.json)
        return await self.query_list(request, param.list_select)

    async def put(self, request: Request):
        """
        增加
        """
        po = UserGroupPO()
        userId = self.getUserId(request)

        async def before(po: UserGroupPO, session: AsyncSession, request):
            exist = await db_tools.exist(session,  UserGroupPO.code.__eq__(po.code), column=UserGroupPO.id)
            if exist:
                return JSON_util.response(Result.fail(message=f"'{po.code}'已存在！"))
        return await self.add(request, po, userId=userId, beforeFun=before)

    def patch(self, request: Request):
        return text("I am patch method")


class user_group_view(AuthMethodView):
    routePath = "/<pk:int>"

    async def put(self, request: Request, pk: int):
        """
        编辑
        """
        async def before(oldPo: UserGroupPO, po: UserGroupPO, session: AsyncSession, request):
            exist = await db_tools.exist(session, UserGroupPO.id != oldPo.id, UserGroupPO.code.__eq__(po.code), column=UserGroupPO.id)
            if exist:
                return JSON_util.response(Result.fail(message=f"'{po.code}'已存在！"))
            if po.parentId == oldPo.id:
                return JSON_util.response(Result.fail(message=f"'父节点选择错误！"))

        return await self.edit(request, pk, UserGroupPO, userId=self.getUserId(request), fun=before)

    async def delete(self, request: Request, pk: int):
        """
        删除
        """
        async def before(po: UserGroupPO, session: AsyncSession):
            count = await db_tools.count(session, UserGroupPO.parentId == po.id, column=UserGroupPO.id)
            if count > 0:
                return JSON_util.response(Result.fail(message=f"该'{po.name}'节点下有‘{count}’节点，不能删除！"))
        return await self.remove(request, pk, UserGroupPO, beforeFun=before)



from sanic import Sanic, Blueprint, Request
from co6co_sanic_ext .api import add_routes
from ..view_model.file import FileViews, FolderView
from ..view_model.file.upload import UploadView, UploadQueryView
from ..view_model.file.batch import batchDelView
from ..view_model.file.simple import RenameView, NewFolderView, FileContentView


# 文件管理
file_api = Blueprint("files_manage", url_prefix="/files")
add_routes(file_api, FileViews, FolderView, UploadView, UploadQueryView, batchDelView, RenameView, NewFolderView, FileContentView)




from sanic import Request
from co6co_sanic_ext.utils import JSON_util
from co6co_sanic_ext.model.res.result import Result
from ..base_view import AuthMethodView
from ...model.filters.file_param import FileParam
import os
from co6co.utils import log
import tempfile
from sanic.request.form import File
from co6co.utils.hash import str_md5


def getTempFileName(folder: str, fileName: str):
    """
    获取临时文件名
    """
    fullPath = os.path.join(folder, fileName)
    return str_md5(fullPath)


def merge_chunks(savePath: str, file_name: str, total_chunks: int):
    """
    合并文件块
    """
    if not os.path.exists(savePath):
        os.makedirs(savePath)
    output_file_path = os.path.join(savePath, file_name)
    tempFileName = getTempFileName(savePath, file_name)
    with open(output_file_path, 'wb') as output_file:
        for i in range(1, total_chunks + 1):
            temp_file_path = os.path.join(tempfile.gettempdir(), f'{tempFileName}_part{i}')
            with open(temp_file_path, 'rb') as temp_file:
                output_file.write(temp_file.read())
            os.remove(temp_file_path)  # 删除临时文件


class UploadQueryView(AuthMethodView):
    routePath = "/upload/query"

    async def post(self, request: Request):
        """
        查询已上传的文件块
        {fileName:xxx.xx,totalChunks:1000}
        """
        params: dict = request.json
        fileName: str = params.get("fileName", None)
        totalChunks = params.get("totalChunks", None)
        uploadPath = params.get("uploadPath", None)
        if not fileName:
            return self.response_json(Result.fail(message="缺少文件名"))
        if not totalChunks:
            return self.response_json(Result.fail(message="缺少totalChunks"))
        totalChunks = int(totalChunks)
        uploaded_chunks = []
        tempFileName = getTempFileName(uploadPath, fileName)
        for i in range(1, totalChunks+1):
            temp_file_path = os.path.join(tempfile.gettempdir(), f'{tempFileName}_part{i}')
            if os.path.exists(temp_file_path):
                uploaded_chunks.append(i)
            else:
                break
        finshed = len(uploaded_chunks) == totalChunks
        if finshed:
            merge_chunks(uploadPath, fileName, totalChunks)
        return self.response_json(Result.success({'uploadedChunks': uploaded_chunks, "finshed": finshed}))


class UploadView(AuthMethodView):
    routePath = "/upload"

    async def put(self, request: Request):
        """
        上传 chunk
        index: 从1开始
        """
        try:
            file: File = request.files.get('file')
            index = int(request.form.get('index'))
            total_chunks = int(request.form.get('totalChunks'))
            file_name = request.form.get('fileName')
            uploadPath = request.form.get('uploadPath')
            if not file or not file_name:
                return self.response_json(Result.fail(message="缺少文件名"))
            tempFileName = getTempFileName(uploadPath, file_name)
            # 保存文件块到临时目录
            temp_file_path = os.path.join(tempfile.gettempdir(), f'{tempFileName}_part{index}')
            await self.save_file(file, temp_file_path)

            # 检查是否所有块都已上传
            if index == total_chunks:
                merge_chunks(uploadPath, file_name, total_chunks)

            return self.response_json(Result.success(message="文件块上传成功"))
        except Exception as e:
            return self.response_json(Result.fail(message="文件块上传失败{}".format(e)))





from sanic.request import Request
from ...model.pos.right import UserPO
from co6co_sanic_ext.utils import JSON_util
from co6co_sanic_ext.model.res.result import Result
from ...services import getCurrentUserId


ObjectExistRoute = "/exist/<code:str>/<pk:int>"


def exist(isExist: bool = True, tableName="用户", name: str = "xxx"):
    if isExist:
        return JSON_util.response(Result.success(data=True, message=f"{tableName}'{name}'已存在。"))
    else:
        return JSON_util.response(Result.success(data=False, message=f"{tableName}'{name}'不已存在。"))




from sanic.response import text
from sanic import Request
from co6co_sanic_ext.utils import JSON_util
from co6co_sanic_ext.model.res.result import Result
from sqlalchemy.ext.asyncio import AsyncSession

from sqlalchemy.sql import Select, Delete

from co6co_db_ext.db_utils import db_tools, DbCallable
from co6co_web_db.model.params import associationParam

from datetime import datetime
from ..aop import exist
from ..base_view import AuthMethodView
from ...model.filters.dict_filter import DictFilter
from ...model.pos.other import sysDictPO, sysDictTypePO
from ...model.enum import dict_state


class DictSelectView(AuthMethodView):
    routePath = "/<dictTypeCode:str>/<category:int>"

    async def get(self, request: Request, dictTypeCode: str, category: int):
        """ 
        获取字典选择
        dictTypeCode: 字典类型代码
        """
        # NameValueFlag = 0,
        # NameValue = 1,
        # NameFlag = 2,
        # All = 999,
        fields = [sysDictPO.id, sysDictPO.name, sysDictPO.flag,  sysDictPO.value]
        if category == 1:
            fields = [sysDictPO.id, sysDictPO.name,  sysDictPO.value]
        if category == 2:
            fields = [sysDictPO.id, sysDictPO.name,  sysDictPO.flag]
        if category == 999:
            fields = [sysDictPO.id, sysDictPO.name, sysDictPO.flag, sysDictPO.value, sysDictPO.desc]

        select = (
            Select(*fields)
            .join(sysDictTypePO, onclause=sysDictPO.dictTypeId == sysDictTypePO.id)
            .filter(sysDictTypePO.code.__eq__(dictTypeCode), sysDictPO.state.__eq__(dict_state.enabled.val))
            .order_by(sysDictPO.order.asc())
        )
        return await self.query_list(request, select,  isPO=False)


class Views(AuthMethodView):
    async def get(self, request: Request):
        """
        字典、字典类型状态
        枚举类型 : dict_state
        """
        return JSON_util.response(Result.success(data=dict_state.to_dict_list()))

    async def post(self, request: Request):
        """
        table数据 
        """
        param = DictFilter()
        return await self.query_page(request, param)

    async def put(self, request: Request):
        """
        增加
        """
        po = sysDictPO()
        userId = self.getUserId(request)

        async def before(po: sysDictPO, session: AsyncSession, request):
            exist = await db_tools.exist(session, sysDictPO.dictTypeId == po.dictTypeId, sysDictPO.value == po.value,   column=sysDictPO.id)
            if exist:
                return JSON_util.response(Result.fail(message=f"'{po.value}'在该字典中已存在！"))
        return await self.add(request, po, userId=userId, beforeFun=before)


class View(AuthMethodView):
    routePath = "/<pk:int>"

    async def put(self, request: Request, pk: int):
        """
        编辑
        """
        async def before(oldPo: sysDictPO, po: sysDictPO, session: AsyncSession, request):
            exist = await db_tools.exist(session, sysDictPO.dictTypeId == po.dictTypeId, sysDictPO.value == po.value, sysDictPO.id != oldPo.id, column=sysDictPO.id)
            if exist:
                return JSON_util.response(Result.fail(message=f"'{po.value}'在该字典中已存在！"))

        return await self.edit(request, pk, sysDictPO, userId=self.getUserId(request), fun=before)

    async def delete(self, request: Request, pk: int):
        """
        删除
        """
        return await self.remove(request, pk, sysDictPO)




from co6co_web_db.view_model import BaseMethodView
from sanic import Request
from co6co_sanic_ext.utils import JSON_util
from co6co_sanic_ext.model.res.result import Result
import uuid
import datetime


class drap_verify_view(BaseMethodView):

    async def post(self, request: Request):
        """
        拖动验证
        """
        json: dict = request.json
        start = json.get("start", 0)
        end = json.get("end", 0)
        data = json.get("data", [])
        start = datetime.datetime.fromtimestamp(start/1000)
        end = datetime.datetime.fromtimestamp(end/1000)
        dif = end-start
        min = datetime.timedelta(milliseconds=60)
        max = datetime.timedelta(seconds=15)
        if dif > min and dif < max:
            s = str(uuid.uuid4())
            _, sDict = self.get_Session(request)
            sDict["verifyCode"] = s
            return JSON_util.response(Result.success(data=s, message=f"验证成功,用时：{dif.total_seconds()}s"))
        else:
            return JSON_util.response(Result.fail(message="验证失败"))




import multiprocessing.managers
from sanic.request import Request
from sqlalchemy.sql import Select

from co6co_db_ext.db_utils import db_tools, DbCallable

from ...model.enum import menu_type
from ...model.pos.right import UserPO, UserRolePO, UserGroupRolePO, menuPO, MenuRolePO
from multiprocessing.managers import DictProxy
from co6co_web_db .services.cacheManage import CacheManage
from co6co.utils import log
from ...services.baseCache import BaseCache


class AuthonCacheManage(BaseCache):

    def __init__(self, request: Request) -> None:
        super().__init__(request)
        pass

    @property
    def _currentRoleKey(self):
        return f'roleids_{self.userId}'

    @property
    def _currentRolevalieKey(self):
        return f'roleids_{self.userId}_valid'

    @property
    async def currentRoles(self) -> list[int]:
        """
        当前用户角色
        """
        if self._currentRoleKey in self.cache and self.cache[self._currentRolevalieKey]:
            if len(self.cache[self._currentRoleKey]) == 0:
                log.warn(f"請查看用戶userId:{self.userId}是否已經關聯角色！！")
            return self.cache[self._currentRoleKey]
        else:
            await self.queryUserRoles()
            return self.cache[self._currentRoleKey]

    def setRolesInvalid(self):
        """
        设置角色缓存无效
        """
        for key in self.cache.keys():
            if "roleids_" in key and '_valid' in key:
                self.cache[key] = False

    async def queryUserRoles(self):
        """
        查询当前用户的所拥有的角色
        结果放置在cache中
        """
        callable = DbCallable(self.session)

        async def exe(session):
            userRolesSelect = (
                Select(UserRolePO.roleId).filter(UserRolePO.userId == UserPO.id, UserPO.id == self.userId)
            )
            userGroupRolesSelect = (
                Select(UserGroupRolePO.roleId).filter(UserGroupRolePO.userGroupId == UserPO.userGroupId, UserPO.id == self.userId)
            )
            ruleList = await db_tools.execForMappings(session, userRolesSelect.union(userGroupRolesSelect))
            roleList = [d.get("role_id") for d in ruleList]
            self.cache[self._currentRoleKey] = roleList
            self.cache[self._currentRolevalieKey] = True
        await callable(exe)

    @property
    def _menuCacheKey(self):
        return 'api_data'

    @property
    def _menuCacheValueKey(self):
        return 'api_data_valid'

    def setMenuDataInvalid(self):
        """
        设置菜单数据缓存无效
        """
        self.cache[self._menuCacheValueKey] = False

    @property
    async def menuData(self):
        """
        所有菜单数据
        """
        if self._menuCacheKey in self.cache and self.cache[self._menuCacheValueKey]:
            return self.cache[self._menuCacheKey]
        else:
            await self.queryMenus()
            return self.cache[self._menuCacheKey]

    async def queryMenus(self):
        """
        所有菜单存在在缓存
        """
        callable = DbCallable(self.session)

        async def exe(session):
            select = (
                Select(menuPO.id,  menuPO.name,   menuPO.url, menuPO.methods, MenuRolePO.roleId)
                .join(MenuRolePO, onclause=MenuRolePO.menuId == menuPO.id)
                .filter(menuPO.category.__eq__(menu_type.api.val))
                .order_by(menuPO.parentId.asc(), menuPO.order.asc())
            )
            menuList = await db_tools.execForMappings(self.session, select)
            self.cache[self._menuCacheKey] = menuList
            self.cache[self._menuCacheValueKey] = True
        await callable(exe)



from sanic import Sanic, Blueprint, Request
from co6co_sanic_ext .api import add_routes
from ..view_model.resource.path_view import Video_View, Image_View, Poster_View, thumbnail_View
from ..view_model.resource.resource_view import Res_Image_View, Res_Video_View, Res_thumbnail_View, Res_Poster_View
from ..view_model.resource.upload_view import Upload_View, Image_View as upLoad_img_View, Video_View as upload_video_view

# 查看
_api = Blueprint("resource_API")
add_routes(_api, Res_Image_View, Res_Video_View, Res_thumbnail_View, Res_Poster_View)
add_routes(_api, Video_View, Image_View, Poster_View, thumbnail_View)

# 上传
_upload_api = Blueprint("resource_upload_API", url_prefix="/upload")
add_routes(_upload_api, Upload_View, upLoad_img_View, upload_video_view)

res_api = Blueprint.group(_api, _upload_api, url_prefix="/res")



from sanic import Sanic, Blueprint, Request
from .menu import menu_api
from .userGroup import userGroup_api
from .role import role_api
from .user import user_api
from .view import view_api
from .dict import dict_api
from .config import config_api
from .resource import res_api
from .file import file_api
from .verify import verify_api

permissions_api = Blueprint.group(
    view_api, menu_api, userGroup_api, role_api, user_api, dict_api, config_api, res_api, file_api, verify_api)




from sanic import Sanic, Blueprint, Request
from ..view_model.dict.dictTypeView import DictTypeViews, DictTypeView, DictTypeExistView
from ..view_model.dict.dictView import Views, View, DictSelectView
from co6co_sanic_ext.api import add_routes

dict_api = Blueprint("dict_API", url_prefix="/dict")
add_routes(dict_api, Views, View, DictSelectView)
add_routes(dict_api,  DictTypeViews, DictTypeView, DictTypeExistView)





from sanic import Sanic, Blueprint, Request
from ..view_model.ui.menu_view import ui_tree_view
from ..view_model.ui.session_view import Session_View
from co6co_sanic_ext.api import add_routes

view_api = Blueprint("view_API", url_prefix="/view")
add_routes(view_api, ui_tree_view, Session_View)
'''
menu_api.add_route(menus_view.as_view(),"/",name=menus_view.__name__) 
menu_api.add_route(menu_tree_view.as_view(),"/tree",name=menu_tree_view.__name__) 
menu_api.add_route(menu_view.as_view(),"/<pk:int>",name=menu_view.__name__) 
'''




from sanic import Request
from sqlalchemy.orm import joinedload
from sqlalchemy.sql import Select
from co6co.utils import log
from co6co_web_db.services.cacheManage import CacheManage
from ..model.pos.right import UserPO
from co6co_db_ext.db_utils import DbCallable, db_tools
from . import getCurrentUserId, getSecret, generateCode
from co6co_db_ext.db_session import db_service
from sqlalchemy.ext.asyncio import AsyncSession
from co6co_web_db.view_model import get_one
from ..model.enum import user_state
from co6co_web_db.services.jwt_service import setCurrentUser
from ..services import queryUerByAccessToken
from sqlalchemy.ext.asyncio import AsyncSession


class BaseCache(CacheManage):

    request = Request

    def __init__(self, request: Request) -> None:
        self.request = request
        super().__init__(request.app)

    @property
    def userId(self):
        """
        当前用户ID
        """
        # 微信认证中 userid可能未挂在上去
        return getCurrentUserId(self.request)


class AccessTokenCache(BaseCache):
    """
    令牌缓存
    """

    def __init__(self, request: Request) -> None:
        super().__init__(request)

    async def validAccessToken(self, token: str) -> bool:
        """
        验证token是否有效
        :param token: 令牌
        :return: bool
        """
        result = self.getCache(token)
        if result:
            await setCurrentUser(self.request, result)
            return True

        # select = Select(UserPO).filter(UserPO.password.__eq__(token), UserPO.state.in_([user_state.enabled]))
        # user: UserPO = await db_tools.execForPo(self._session, select, remove_db_instance_state=False)
        async with self.session:  # , self.session.begin() begin会开启事务
            user: UserPO = await queryUerByAccessToken(self._session, token)
            if user == None:
                log.warn("query {} accessToken is NULL".format(token))
                return False
            else:
                data = user.to_jwt_dict()
                self.setCache(token, user.to_jwt_dict())
                await setCurrentUser(self.request, data)
                return True




from sanic.response import text
from sanic import Request
from co6co_sanic_ext.utils import JSON_util
from co6co_sanic_ext.model.res.result import Result
from sqlalchemy.ext.asyncio import AsyncSession

from sqlalchemy.sql import Select, Delete

from co6co_db_ext.db_utils import db_tools, DbCallable
from co6co_web_db.model.params import associationParam

from datetime import datetime
from .base_view import AuthMethodView
from .aop import exist, ObjectExistRoute
from ..model.pos.right import RolePO, MenuRolePO, UserRolePO, UserGroupRolePO, menuPO
from ..model.filters.role_filter import role_filter
from .aop.api_auth import userRoleChanged


class roles_ass_exist_view(AuthMethodView):
    routePath = ObjectExistRoute

    async def get(self, request: Request, code: str, pk: int = 0):
        result = await self.exist(request, RolePO.code == code,
                                  RolePO.id != pk, column=RolePO.id)
        return exist(result, "角色编码", code)


class roles_ass_view(AuthMethodView):
    routePath = "/association/<roleId:int>"

    async def post(self, request: Request, roleId: int):
        """
        获取角色关联菜单
        """
        subSelect = Select(MenuRolePO.menuId, MenuRolePO.roleId).filter(
            MenuRolePO.roleId == roleId).subquery()
        select = (
            Select(menuPO.id, menuPO.name, menuPO.code, menuPO.parentId,
                   subSelect.c.roleId.label("associatedValue"))
            .outerjoin_from(menuPO, subSelect, onclause=subSelect.c.menuId == menuPO.id, full=False)
            .order_by(menuPO.parentId.asc())
        )
        return await self.query_tree(request, select, rootValue=0, pid_field='parentId', id_field="id", isPO=False)

    @userRoleChanged
    async def put(self, request: Request, roleId: int):
        """
        保存角色关联菜单
        """
        param = associationParam()
        param.__dict__.update(request.json)
        userId = self.getUserId(request)

        sml = (
            Delete(MenuRolePO).filter(MenuRolePO.roleId == roleId, MenuRolePO.menuId .in_(param.remove))
        )

        async def createPo(_, menuId: int):
            po = MenuRolePO()
            po.menuId = menuId
            po.roleId = roleId
            return po
        return await self.save_association(request, userId, sml, createPo)


class roles_view(AuthMethodView):
    async def get(self, request: Request):
        """
        树形选择下拉框数据
        selectTree :  el-Tree
        """
        select = (
            Select(RolePO.id, RolePO.name, RolePO.code)
            .order_by(RolePO.order.asc())
        )
        return await self.query_list(request, select,  isPO=False)

    async def post(self, request: Request):
        """
        table数据 
        """
        param = role_filter(request)
        await param.init()
        param.__dict__.update(request.json)
        return await self.query_page(request, param)

    async def put(self, request: Request):
        """
        增加
        """
        po = RolePO()
        userId = self.getUserId(request)

        async def before(po: RolePO, session: AsyncSession, request):
            exist = await db_tools.exist(session,  RolePO.code.__eq__(po.code), column=RolePO.id)
            if exist:
                return JSON_util.response(Result.fail(message=f"'{po.code}'已存在！"))
        return await self.add(request, po, userId=userId, beforeFun=before)

    def patch(self, request: Request):
        return text("I am patch method")


class role_view(AuthMethodView):
    routePath = "/<pk:int>"

    async def put(self, request: Request, pk: int):
        """
        编辑
        """
        async def before(oldPo: RolePO, po: RolePO, session: AsyncSession, request):
            exist = await db_tools.exist(session, RolePO.id != oldPo.id, RolePO.code.__eq__(po.code), column=RolePO.id)
            if exist:
                return JSON_util.response(Result.fail(message=f"'{po.code}'已存在！"))

        return await self.edit(request, pk, RolePO, userId=self.getUserId(request), fun=before)

    async def delete(self, request: Request, pk: int):
        """
        删除
        """
        async def before(po: RolePO, session: AsyncSession):
            count = await db_tools.count(session, MenuRolePO.roleId == po.id, column=RolePO.id)
            if count > 0:
                return JSON_util.response(Result.fail(message=f"该'{po.name}'角色关联了‘{count}’个菜单，不能删除！"))
            count = await db_tools.count(session, UserGroupRolePO.roleId == po.id, column=RolePO.id)
            if count > 0:
                return JSON_util.response(Result.fail(message=f"该'{po.name}'角色关联了‘{count}’个用户组，不能删除！"))
            count = await db_tools.count(session, UserRolePO.roleId == po.id, column=RolePO.id)
            if count > 0:
                return JSON_util.response(Result.fail(message=f"该'{po.name}'角色关联了‘{count}’个用户，不能删除！"))

        return await self.remove(request, pk, RolePO, beforeFun=before)




from sanic.response import text
from sanic import Request
from co6co_sanic_ext.utils import JSON_util
from co6co_sanic_ext.model.res.result import Result
from sqlalchemy.ext.asyncio import AsyncSession

from sqlalchemy.sql import Select, Delete

from co6co_db_ext.db_utils import db_tools, DbCallable
from co6co_web_db.model.params import associationParam

from datetime import datetime
from ..base_view import AuthMethodView
from ..aop import exist, ObjectExistRoute
from ...model.filters.config_filter import Filter
from ...model.pos.other import sysConfigPO
from ..aop.config_aop import ConfigEntry
from ...services .configCache import ConfigCache


class ConfigView(AuthMethodView):
    """
    通过代码获取配置
    """
    routePath = "/<code:str>"

    async def get(self, request: Request, code: str):
        """ 
        获取配置
        code: 配置代码
        """
        select = (
            Select(sysConfigPO.name, sysConfigPO.code,
                   sysConfigPO.value, sysConfigPO.remark)
            .filter(sysConfigPO.code.__eq__(code))
        )
        return await self.query_mapping(request, select, oneRecord=True)


class ConfigByCacheView(AuthMethodView):
    """
    通过代码获取配置
    """
    routePath = "/Cache/<code:str>"

    async def post(self, request: Request, code: str):
        """ 
        获取配置
        code: 配置代码

        return str,配置值
        """
        cache = ConfigCache(request)
        config = cache.getConfig(code)
        if not config:
            config = await cache .queryConfig(code)
        if config is None:
            return self.response_json(Result.fail(message="未能获取配置，请检查是否有配置项'{}',或检查配置的json对象是否正确".format(code)))
        return self.response_json(Result.success(config))


class ExistView(AuthMethodView):
    routePath = ObjectExistRoute

    async def get(self, request: Request, code: str, pk: int = 0):
        result = await self.exist(request, sysConfigPO.code == code, sysConfigPO.id != pk)
        return exist(result, "配置code", code)


@ConfigEntry
async def configValueChange(request: Request, code: str, value: str):
    return value


class Views(AuthMethodView):
    async def post(self, request: Request):
        """
        table数据 
        """
        param = Filter()
        return await self.query_page(request, param)

    async def put(self, request: Request):
        """
        增加
        """
        po = sysConfigPO()
        userId = self.getUserId(request)

        async def before(po: sysConfigPO, session: AsyncSession, request):
            exist = await db_tools.exist(session,  sysConfigPO.code.__eq__(po.code), column=sysConfigPO.id)
            if exist:
                return JSON_util.response(Result.fail(message=f"'{po.code}'已存在！"))

        async def after(po: sysConfigPO, session, request):
            await configValueChange(request, po.code, po.value)

        return await self.add(request, po, userId=userId, beforeFun=before, afterFun=after)


class View(AuthMethodView):
    routePath = "/<pk:int>"

    async def put(self, request: Request, pk: int):
        """
        编辑
        """
        async def before(oldPo: sysConfigPO, po: sysConfigPO, session: AsyncSession, request):
            exist = await db_tools.exist(session, sysConfigPO.id != oldPo.id, sysConfigPO.code.__eq__(po.code), column=sysConfigPO.id)
            if exist:
                return JSON_util.response(Result.fail(message=f"'{po.code}'已存在！"))
            await configValueChange(request, po.code, po.value)

        return await self.edit(request, pk, sysConfigPO, userId=self.getUserId(request), fun=before)

    async def delete(self, request: Request, pk: int):
        """
        删除
        """
        return await self.remove(request, pk, sysConfigPO)




import os
import shutil


def delFileOrFolder(path: str):
    """
    删除文件或目录
    """
    if os.path.isfile(path):
        os.unlink(path)
    elif os.path.isdir(path):
        # os.rmdir(path) 删除空文件夹
        shutil.rmtree(path)



from setuptools import setup

from setuptools.command.sdist import sdist
import subprocess
import sys
import co6co.setupUtils as setupUtils
# try:
#    from co6co import setupUtils
# except ImportError:
#    subprocess.check_call([sys.executable, "-m", "pip", "install", "co6co>=0.0.25"])
#    from co6co import setupUtils

version = setupUtils.get_version(__file__)
packageName, packages = setupUtils.package_name(__file__)
long_description = setupUtils.readme_content(__file__)


setup(
    name=packageName,
    version=version,
    description="web permissionsAPI",
    packages=packages,

    long_description=long_description,
    long_description_content_type='text/markdown',
    classifiers=setupUtils.get_classifiers(),
    include_package_data=True, zip_safe=True,
    # 依赖哪些模块
    install_requires=['requests', "co6co>=0.0.29", "co6co.sanic_ext>=0.0.9", "co6co.web-db>=0.0.14", "opencv-python==4.10.0.82", "numpy==1.26.4", "Pillow>=10.1.0"],

    # package_dir= {'utils':'src/log','main_package':'main'},#告诉Distutils哪些目录下的文件被映射到哪个源码
    author='co6co',
    author_email='co6co@qq.com',
    url="http://github.com/co6co",
    data_file={
        ('', "*.txt"),
        ('', "*.md"),
    },
    package_data={
        '': ['*.txt', '*.md'],
        'bandwidth_reporter': ['*.txt']
    }, cmdclass={
        'sdist': setupUtils.CustomSdist
    }
)




from ...services.configCache import get_upload_path
from .utils import resize_image, screenshot, getTempFileName
from co6co .utils import log
from ..base_view import AuthMethodView
import json
import io
import os
from PIL import Image
from io import BytesIO
import requests
from sanic import Request
from sanic.response import text, raw, empty, file, file_stream, ResponseStream
from ...model.pos.resource import resourcePO
from co6co_db_ext.db_utils import QueryOneCallable
from sqlalchemy.sql import Select
from sanic.request.form import File
from co6co.utils import hash, getDateFolder
from ...model.enum import resource_category
from datetime import datetime
import uuid
from co6co_sanic_ext.model.res.result import Result


class FileResult:
    fullPath: str
    path: str
    hash: str
    name: str
    size: int

    def toPo(self, category: resource_category, subCategory: int = 0):
        po = resourcePO()
        po .uid = uuid.uuid4()
        po .category = category.val
        po .subCategory = subCategory
        po .hash = self.hash
        po .url = self.path
        po .metaName = self.name
        po .size = self.size
        po.createTime = datetime.now()
        return po


class resource_baseView(AuthMethodView):
    async def getRersourcePath(self, request: Request, path):
        """
        获取资源路径

        @param request   获取上传根目录
        @param path  eg. /upload/2022/01/01/123.jpg

        return fullPath
        """
        uploadRoot = await get_upload_path(request)
        fullPath = os.path.join(uploadRoot, path[1:])
        return os.path.abspath(fullPath)

    async def getLocalPathById(self, request: Request, pk: int) -> str:
        """
        通过 id 获取本地路径
        return fullPath
        """
        call = QueryOneCallable(self.get_db_session(request))
        data = await call(Select(resourcePO.url).filter(resourcePO.id == pk), isPO=False)
        if data != None:
            return await self.getRersourcePath(request, data["url"])
        return None

    async def getLocalPath(self, request: Request) -> str:
        path = ""
        for k, v in request.query_args:
            if k == "path":
                path = v
        if path.startswith("http"):
            return path
        return await self.getRersourcePath(request, path)

    async def screenshot(self, fullPath: str, w: int = 208, h: int = 117, isFile: bool = True):
        """
        视频截图
        视频第一帧作为 poster
        """
        if fullPath.startswith('http') or os.path.exists(fullPath):
            isFile = not fullPath.startswith('http')
            tempPath = await screenshot(fullPath, w, h, isFile=isFile, useBytes=True)
            if tempPath == None:
                return empty(status=404)
            return raw(tempPath,  status=200, headers=None,  content_type="image/jpeg")
        return empty(status=404)

    async def readHttpImage(self, url):
        """
        从 url 中获取 图片
        """
        data = requests.get(url)
        if data.status_code == 200:
            data = BytesIO(data.content)
            im = Image.open(data)
            return im
        return None

    async def readLocalImage(path):
        if os.path.exists(path):
            im = Image.open(path)
            return im
        return None

    async def response_local_file(self, request: Request, fullPath: str):
        """
        响应本地文件
        """
        filePath = await self.getRersourcePath(request, fullPath)
        if os.path.exists(filePath):
            return await file(filePath)

    async def screenshot_image(self, fullPath: str, w: int = 208, h: int = 117):
        """ 
        @param fullPath 完整路径 url 或者本地路径
        略缩图
        """
        try:
            im = None
            if fullPath.startswith('http'):
                im = await self.readHttpImage(fullPath)
            elif os.path.exists(fullPath):
                im = await self.readLocalImage(fullPath)
            if im != None:
                bytes = io.BytesIO()
                im.thumbnail((w, h))
                im.save(bytes, "PNG")
                return raw(bytes.getvalue(),  status=200, headers=None,  content_type="image/jpeg")
            return empty(status=404)
        finally:
            if im != None:
                im.close()

    async def saveFile(self, request: Request):
        """
        保存文件到本地
        @param request  request.files.get("file") 
        return FileResult
        """
        try:
            fullPath = None
            basePath = await get_upload_path(request)
            if "file" not in request.files:
                return self.response_json(Result.fail("No file part in the request"))
            file: File = request.files.get("file")

            size = len(file.body)  # await self. readFileSize(file)
            _, file_extension = os.path.splitext(file.name)
            fileName = getDateFolder(format='%Y-%m-%d-%H-%M-%S')
            fullPath, path = self.getFullPath(basePath, "{}{}".format(fileName, file_extension))
            await self.save_file(file, fullPath)
            result = FileResult()
            result.name = file.name
            result.hash = hash.file_sha256(fullPath)
            result.size = size
            result.fullPath = fullPath
            result.path = path
            return result
        except Exception as e:
            log.warn("保存文件失败：", fullPath, e)
            return self.response_json(Result.fail(message=e))




from sanic import Request
from sanic.response import file
from sanic.request.form import File
from sqlalchemy.orm import selectinload
from co6co.utils import log
from . import resource_baseView, FileResult
from ...services.configCache import get_upload_path
from co6co_sanic_ext.model.res.result import Result
from ...model.pos.resource import resourcePO, userResourcePO
from ...model.enum import resource_category

import os
from datetime import datetime
from co6co_db_ext.db_utils import DbCallable, db_tools

from sqlalchemy.sql import Select
from sqlalchemy.ext.asyncio import AsyncSession
from co6co_web_db.view_model import errorLog
from co6co.utils.tool_util import get_current_function_name


class Upload_View(resource_baseView):
    routePath = "/file"

    async def readFileSize(self, file: File):
        """
        获取文件大小
        """
        file_size = 0
        chunk_size = 1024
        while True:
            chunk = file.body.read(chunk_size)
            if not chunk:
                break
            file_size += len(chunk)
        file.body.seek(0)
        return file_size

    async def saveDb(self, request: Request,  param: FileResult, category: resource_category) -> int:
        call = DbCallable(self.get_db_session(request))

        async def exec(session: AsyncSession):
            select = (Select(resourcePO).filter(resourcePO.hash.__eq__(param.hash)) .options(
                selectinload(resourcePO.userResourceList)
            ))

            dbPo: resourcePO = await db_tools.execForPo(session, select, remove_db_instance_state=False)
            userPo = userResourcePO()
            userPo.ownUserId = self.getUserId(request)
            userPo.createTime = datetime.now()
            userPo.name = param.name
            resourceId = None
            # 数据库中已经存在
            if dbPo != None:
                basePath = await get_upload_path(request)
                oldPath = os.path.join(basePath, dbPo.url[1:])
                if os.path.exists(oldPath) and os.path.exists(param.fullPath):
                    os.remove(param.fullPath)
                if not os.path.exists(oldPath):
                    dbPo.url = param.path

                dbPo.userResourceList.append(userPo)
                resourceId = dbPo.id
            else:
                po = param.toPo(category)
                po.userResourceList = [userPo]
                session.add(po)
                await session.flush()
                resourceId = po.id
            return resourceId
        return await call(exec)

    async def putFile(self, request: Request, category: resource_category):
        """
        上传文件
        """
        try:
            result = await self.saveFile(request)
            if type(result) != FileResult:
                return result
            result: FileResult = result
            resourceId = await self.saveDb(request, result, category)
            return self.response_json(Result.success(data={"resourceId": resourceId, "path": result.path}))
        except Exception as e:
            errorLog(request, self.__class__, get_current_function_name())
            return self.response_json(Result.fail(message="上传失败:{}".format(e)))

    async def put(self, request: Request):
        """
        上传文件
        """
        return await self.putFile(request, resource_category.file)


class Image_View(Upload_View):
    routePath = "/img"

    async def put(self, request: Request):
        """
        上传图片
        """
        return await self.putFile(request, resource_category.image)


class Video_View(Upload_View):
    routePath = "/video"

    async def put(self, request: Request):
        """
        上传视频
        """
        return await self.putFile(request, resource_category.video)




from sanic.response import text
from sanic import Request
from sanic.response import file, file_stream, json, raw
from co6co_sanic_ext.utils import JSON_util
from co6co_sanic_ext.model.res.result import Result
from ..base_view import AuthMethodView
from ...model.filters.file_param import FileParam
from ...services import fileService
import os
import datetime
from co6co.utils import log
import tempfile
import shutil


class File:
    isFile: bool
    name: str
    path: str
    right: str
    date: datetime.datetime
    size: int

    def __init__(self):
        self.isFile = None
        self.name = None
        self.path = None
        self.right = None
        self.updateTime = None
        self.size = None
        pass

    def __init__(self, root, name):
        self.name = name
        self.path = os.path.join(root,   name)
        self.path = os.path.abspath(self.path)
        self.isFile = os.path.isfile(self.path)
        if self.isFile:
            self.size = os.path.getsize(self.path)
        self.right = None
        self.updateTime = datetime.datetime.fromtimestamp(os.path.getmtime(self.path))
        pass


class FolderView(AuthMethodView):
    routePath = "/zip"

    async def head(self, request: Request):
        """
        文件夹打包
        """
        try:
            args = self.usable_args(request)
            filePath = args.get("path")
            if os.path.isfile(filePath):
                raise Exception("该方法不支持文件")
            timeStr = datetime.datetime.now().strftime("%Y-%m-%d_%H%M%S")
            fileName = "{}_{}.zip".format(os.path.basename(filePath), timeStr)
            # cache:Cache= request.app.ctx.Cache
            # uid=uuid.uuid4()
            # cache.add(uid,fileName)
            zipFilePath = os.path.join('.', fileName)
            await self.zip_directory(filePath, zipFilePath)
            id = request.headers.get("session")
            request.app.ctx.data = {id: {filePath: zipFilePath}}
            return await self.response_size(zipFilePath)
        except Exception as e:
            return self.response_json(Result.fail("压缩文件出错:{}".format(e)))

    async def get(self, request: Request):
        try:
            args = self.usable_args(request)
            key = args.get("path")
            sessionId = request.headers.get("session")
            filePath = request.app.ctx.data[sessionId].get(key)
            _, end, file_size = self.parseRange(request, filePath=filePath)
            return await self.get_file_partial(request, filePath)
        finally:
            if filePath != None and os.path.exists(filePath) and file_size-1 == end:
                # log.warn("删除文件.", filePath)
                os.remove(filePath)
                # os.unlink(zipFilePath)
                cache: dict = request.app.ctx.data[sessionId]
                log.warn(cache, key, cache.keys)
                if key in cache:
                    cache.pop(key)
                if {} == cache:
                    cache: dict = request.app.ctx.data
                    if sessionId in cache:
                        cache.pop(sessionId)


class FileViews(AuthMethodView):

    async def head(self, request: Request):
        """
        文件或目录大小
        """
        args = self.usable_args(request)
        filePath = args.get("path")
        return await self.response_size(filePath)

    async def get(self, request: Request):
        """
        下载文件
        """
        args = self.usable_args(request)
        filePath = args.get("path")
        if os.path.isdir(filePath):
            raise ValueError("该方法不支持文件夹下载")
        return await self.get_file_partial(request, filePath)

    async def post(self, request: Request):
        """
        列表
        """
        param = FileParam()
        param.__dict__.update(request.json)
        if param.root == None:
            param.root = "/"
        if param.root.endswith(":"):
            param.root = param.root+os.sep

        def filter(x): return param.name == None or param.name in x
        list = os.listdir(param.root)
        result = []
        for s in list:
            if filter(s):
                folder = File(param.root, s)
                result.append(folder)
        return self.response_json(Result.success({"root": param.root, "res": result}))

    async def delete(self, request: Request):
        args = self.usable_args(request)
        filePath = args.get("path")
        if not os.path.exists(filePath):
            return self.response_json(Result.success(message=f"路径：{filePath},不存在！"))
        fileService.delFileOrFolder(filePath)
        return self.response_json(Result.success(message=f"删除'{filePath}'完成！"))

    async def put(self, request: Request):
        # 创建一个临时文件用于保存上传的数据
        temp_file = tempfile.NamedTemporaryFile(delete=False)
        try:
            # 定义一个异步生成器来处理文件流
            async def process_data(stream):
                async for data in stream:
                    temp_file.write(data)

            # 调用 request.stream 并传入生成器
            await request.stream(process_data)
            # 这里可以添加更多处理逻辑，比如保存文件到永久位置等
            return self.response_json(Result.success())

        finally:
            # 确保无论如何都会执行的清理代码
            temp_file.close()
            # 如果需要，在这里删除临时文件
            os.unlink(temp_file.name)  # 删除临时文件
